import cv2
import numpy as np

class DetectaMovVideo:
    def __init__(self):
        self.prevFrame = None
        self.prevCenter = None

    def detectarMovimiento(self, videoPath):
        resultados = []

        # Abrir el video
        capture = cv2.VideoCapture(videoPath)
        if not capture.isOpened():
            resultados.append("No se pudo abrir el video")
            return resultados

        while True:
            ret, frame = capture.read()
            if not ret:
                break

            # Convertir a escala de grises y difuminar
            gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            gray = cv2.GaussianBlur(gray, (21, 21), 0)

            # Si es el primer cuadro, guardar y continuar
            if self.prevFrame is None:
                self.prevFrame = gray
                continue

            # Calcular la diferencia absoluta entre el cuadro actual y el anterior
            frameDelta = cv2.absdiff(self.prevFrame, gray)
            thresh = cv2.threshold(frameDelta, 25, 255, cv2.THRESH_BINARY)[1]

            # Encontrar contornos en el umbral
            contours, _ = cv2.findContours(thresh.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

            for contour in contours:
                if cv2.contourArea(contour) < 500:
                    continue

                # Calcular el centroide del contorno
                M = cv2.moments(contour)
                cX = int(M["m10"] / M["m00"])
                cY = int(M["m01"] / M["m00"])
                center = (cX, cY)

                # Si es el primer centro, guardar y continuar
                if self.prevCenter is None:
                    self.prevCenter = center
                    continue

                # Calcular la dirección del movimiento
                dx = center[0] - self.prevCenter[0]
                dy = center[1] - self.prevCenter[1]

                if abs(dx) > 2 and abs(dy) > 2:
                    if dx > 0 and dy > 0:
                        resultados.append("\n movimiento en diagonal hacia abajo y a la derecha")
                    elif dx > 0 and dy < 0:
                        resultados.append("\n movimiento en diagonal hacia arriba y a la derecha")
                    elif dx < 0 and dy > 0:
                        resultados.append("\n movimiento en diagonal hacia abajo y a la izquierda")
                    elif dx < 0 and dy < 0:
                        resultados.append("\n movimiento en diagonal hacia arriba y a la izquierda")

                elif abs(dx) > 2:
                    resultados.append("\n movimiento " + ("hacia la derecha" if dx > 0 else "hacia la izquierda"))
                elif abs(dy) > 2:
                    resultados.append("\n movimiento " + ("hacia abajo" if dy > 0 else "hacia arriba"))

                self.prevCenter = center

            self.prevFrame = gray

        capture.release()
        return resultados


import cv2
import numpy as np

class DetectaMovCamara:
    def __init__(self):
        self.prevFrame = None
        self.prevCenter = None

    def detectarMovimiento(self):
        # Abrir la cámara
        capture = cv2.VideoCapture(0)
        if not capture.isOpened():
            print("No se pudo abrir la cámara")
            return

        while True:
            ret, frame = capture.read()
            if not ret:
                break

            gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            gray = cv2.GaussianBlur(gray, (21, 21), 0)

            if self.prevFrame is None:
                self.prevFrame = gray
                continue

            frameDelta = cv2.absdiff(self.prevFrame, gray)
            thresh = cv2.threshold(frameDelta, 25, 255, cv2.THRESH_BINARY)[1]

            contours, _ = cv2.findContours(thresh.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
            for contour in contours:
                if cv2.contourArea(contour) < 500:
                    continue
                (x, y, w, h) = cv2.boundingRect(contour)
                center = (x + w / 2, y + h / 2)

                if self.prevCenter is None:
                    self.prevCenter = center
                    continue

                dx = center[0] - self.prevCenter[0]
                dy = center[1] - self.prevCenter[1]

                if abs(dx) > 2 and abs(dy) > 2:
                    direction = "movimiento en diagonal hacia " + ("abajo y a la derecha" if dx > 0 and dy > 0 else "arriba y a la derecha" if dx > 0 else "abajo y a la izquierda" if dy > 0 else "arriba y a la izquierda")
                elif abs(dx) > 2:
                    direction = "movimiento hacia la " + ("derecha" if dx > 0 else "izquierda")
                elif abs(dy) > 2:
                    direction = "movimiento hacia " + ("abajo" if dy > 0 else "arriba")

                print(direction)
                self.prevCenter = center

            self.prevFrame = gray
        capture.release()
        return direction
    
    
    def mostrarCamara(self):
        print ("Mostrando camara")
        #cv2.destroyAllWindows()


import unittest
from DetectaMovCamara import DetectaMovCamara

class TestDetectaMovCamara(unittest.TestCase):
    def setUp(self):
        self.detector = DetectaMovCamara()

    def test_detectarMovimiento(self):
        # Ejecutar la función a probar
        try:
            self.detector.mostrarCamara()
            self.detector.detectarMovimiento()
        except Exception as e:
            self.fail(f'detectarMovimiento raised Exception unexpectedly: {e}')

if __name__ == '__main__':
    unittest.main()


import cv2

class DetectaFormas:
    def detectarFormas(self, rutaImagen):
        # Cargar la imagen
        src = cv2.imread(rutaImagen)

        # Convertir la imagen a escala de grises
        gray = cv2.cvtColor(src, cv2.COLOR_BGR2GRAY)

        # Aplicar desenfoque para suavizar la imagen
        gray = cv2.blur(gray, (3, 3))

        # Detectar los bordes usando Canny
        edges = cv2.Canny(gray, 50, 150)

        # Encontrar contornos
        contours, hierarchy = cv2.findContours(edges, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

        formas = []

        # Iterar sobre los contornos detectados
        for contour in contours:
            # Calcular el perímetro del contorno
            perimeter = cv2.arcLength(contour, True)

            # Aproximar el contorno a una forma más simple
            approx = cv2.approxPolyDP(contour, 0.02 * perimeter, True)

            # Clasificar la forma del contorno
            if len(approx) == 2:
                formas.append("linea")
            if len(approx) == 3:
                formas.append("triangulo")
            elif len(approx) == 4:
                formas.append("cuadrado")
            elif len(approx) == 5:
                formas.append("pentagono")
            elif len(approx) == 6:
                formas.append("hexagono")
            elif len(approx) == 7:
                formas.append("heptagono")
            elif len(approx) >= 8:
                formas.append("circulo")
            else:
                formas.append("forma desconocida")

        return formas